/* ftpq: add.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

int add_to_queue (char **argv) {

	char *svrdir = NULL;
	char *usrdir = NULL;
	char *usrdet = NULL;
	char *splpath = NULL;
	char *subdir = NULL; 
	char *qname = NULL;
	FILE *usrptr; 
	FILE *qptr;
	int curdepth;
	int spldepth;
	int strlngth; 


	/* Create a subdirectory in the spool directory for the */
	/* server+port combination */
	/* FIXME: needs error handling on the mkdir () call. */
	svrdir = memalloc (strlen (SPOOLDIR) + strlen (gv.host) + 8);
	if (SPOOLDIR [strlen(SPOOLDIR) - 1] == '/') 
		sprintf (svrdir, "%s%s+%d", SPOOLDIR, gv.host, gv.port);
	else
		sprintf (svrdir, "%s/%s+%d", SPOOLDIR, gv.host, gv.port);
	mkdir (svrdir, (mode_t) 0777);

	/* Now create a subdirectory to it for the user */
	usrdir = memalloc (strlen (svrdir) + strlen (gv.login) + 2);
	sprintf (usrdir, "%s/%s", svrdir, gv.login);
	mkdir (usrdir, (mode_t) 0700);

	/* Update the user details file with password and/or account */
	usrdet = memalloc (strlen (usrdir) + 6);
	sprintf (usrdet, "%s/user", usrdir);
	usrptr = fileopen (usrdet, FO_WRITE | FO_NEED);
	
	if (gv.passwd != NULL)
		fprintf (usrptr, "p: %s\n", gv.passwd);
	if (gv.acct != NULL)
		fprintf (usrptr, "a: %s\n", gv.acct);

	fclose (usrptr);
	chmod (usrdet, (mode_t) 0600);

	/* Now we need to create a "mirror" of the server filesystem */

	if (gv.mode & (MODE_PUT|MODE_DEL|MODE_RMDIR|MODE_MKDIR)) {
		splpath = memalloc (strlen (usrdir) + strlen (gv.dstpath) + 7);
		if (gv.dstpath[0] == '/')
			sprintf (splpath, "%s/root%s", usrdir, gv.dstpath);
		else
			sprintf (splpath, "%s/home/%s", usrdir, gv.dstpath);
	}

	else if (gv.mode & MODE_GET) {
		splpath = memalloc (strlen (usrdir) + strlen (gv.srcpath) + 7);
		if (gv.srcpath[0] == '/')
			sprintf (splpath, "%s/root%s", usrdir, gv.srcpath);
		else
			sprintf (splpath, "%s/home/%s", usrdir, gv.srcpath);
	}
	
	spldepth = cnt_subdirs (splpath);

	for (curdepth = cnt_subdirs (usrdir) + 2; curdepth < spldepth + 1; curdepth++) {
		strlngth = path_depth (splpath, curdepth);	
		subdir = memalloc ((size_t) strlngth + 1);
		strncpy (subdir, splpath, strlngth);
		subdir[strlngth] = '\0'; 
		mkdir (subdir, (mode_t) 0700);
		free (subdir);
	}

	if (gv.mode & (MODE_GET | MODE_PUT | MODE_DEL)) 
		qname = splpath; 
	else {
		qname = memalloc (strlen (splpath) + 12);
		sprintf (qname, "%s.ftpq.dirop", splpath);
		free (splpath);
	}

	qptr = fileopen (qname, FO_WRITE | FO_NEED);
	
	fprintf (qptr, "ftpq-%1.2f jobfile\n", VERSION);
	fprintf (qptr, "jo: %ld\n", next_jobid ());
	if (gv.mode & MODE_GET) {
		fprintf (qptr, "lp: %s\n", gv.dstpath);
		fprintf (qptr, "rp: %s\n", gv.srcpath);
	}
	else {
		if (gv.srcpath != NULL)	 
			fprintf (qptr, "lp: %s\n", gv.srcpath);
		fprintf (qptr, "rp: %s\n", gv.dstpath);
	}
	fprintf (qptr, "md: %d\n", gv.mode);
	fprintf (qptr, "mo: %d\n", gv.mail);

	fclose (qptr);

	free (qname);
	free (svrdir);
	free (usrdir);
	free (usrdet);

	return OK;
}


long next_jobid (void) {

	FILE *cntptr = NULL;
	char *cntfile = NULL;
	long jobid = 1L;

	cntfile = memalloc (strlen (SPOOLDIR) + 7);
	if (SPOOLDIR [strlen (SPOOLDIR) - 1] == '/')
		sprintf (cntfile, "%scount", SPOOLDIR);
	else
		sprintf (cntfile, "%s/count", SPOOLDIR);

	cntptr = fileopen (cntfile, FO_READ | FO_NONEED);
	if (cntptr != NULL) {
		fscanf (cntptr, "%ld", &jobid);
		fclose (cntptr);	
	}

	cntptr = fileopen (cntfile, FO_WRITE | FO_NEED);
	fprintf (cntptr, "%ld\n", jobid + 1L);
	fclose (cntptr);	

	free (cntfile);

	return jobid;
}
