/* ftpq: ftpcmd.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

/* Commands taken from RFC959. Unnecessary commands (eg STMT) omitted. */ 

int ftp_user (char *arg, int sockfd) {

	int response;
   	char *send = NULL;

	send = memalloc (strlen (arg) + 8);
	sprintf (send, "USER %s\r\n", arg);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

int ftp_pass (char *arg, int sockfd) {

	int response;
   	char *send = NULL;

	send = memalloc (strlen (arg) + 8);
	sprintf (send, "PASS %s\r\n", arg);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

int ftp_cwd (char *arg, int sockfd) {

	int response;
   	char *send = NULL;

	send = memalloc (strlen (arg) + 7);
	sprintf (send, "CWD %s\r\n", arg);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

int ftp_cdup (int sockfd) {

	return send_cmd ("CDUP\r\n", sockfd, RESP);
}

int ftp_port (int h1, int h2, int h3, int h4, int p1, int p2, int sockfd) {

	int response;
	char send[26];

	sprintf (send, "PORT %d,%d,%d,%d,%d,%d\r\n", h1, h2, h3, h4, p1, p2);
	response = send_cmd (send, sockfd, RESP);
	return response;
}

int ftp_rein (int sockfd) {

	return send_cmd ("REIN\r\n", sockfd, RESP);
}

int ftp_quit (int sockfd) {

	return send_cmd ("QUIT\r\n", sockfd, RESP);
}

int ftp_acct (char *arg, int sockfd) {

	int response;
   	char *send = NULL;

	send = memalloc (strlen (arg) + 8);
	sprintf (send, "ACCT %s\r\n", arg);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

int ftp_modes (int sockfd) {

	return send_cmd ("MODE S\r\n", sockfd, RESP);
}

int ftp_typei (int sockfd) {

	return send_cmd ("TYPE I\r\n", sockfd, RESP);
}

int ftp_struf (int sockfd) {

	return send_cmd ("STRU F\r\n", sockfd, RESP);
}

int ftp_pasv (int sockfd) {

	/* Remember, the host response ends up in the hostresp[] array */

	return send_cmd ("PASV\r\n", sockfd, RESP);
}

int ftp_retr (char *arg, int sockfd) {

	char *send = NULL;
	int response;	

	send = memalloc (strlen (arg) + 8);
	sprintf (send, "RETR %s\r\n", arg);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

int ftp_stor (char *arg, int sockfd) {

	char *send = NULL;
	int response;	

	send = memalloc (strlen (arg) + 8);
	sprintf (send, "STOR %s\r\n", arg);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

int ftp_dele (char *arg, int sockfd) {

	char *send = NULL;
	int response;	

	send = memalloc (strlen (arg) + 8);
	sprintf (send, "DELE %s\r\n", arg);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

int ftp_mkd (char *arg, int sockfd) {

	char *send = NULL;
	int response;	

	send = memalloc (strlen (arg) + 8);
	sprintf (send, "MKD %s\r\n", arg);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

int ftp_rmd (char *arg, int sockfd) {

	char *send = NULL;
	int response;	

	send = memalloc (strlen (arg) + 8);
	sprintf (send, "RMD %s\r\n", arg);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

int ftp_allo (long size, int sockfd) {

	int response;
	char send [20];

	sprintf (send, "ALLO %ld\r\n", size);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	return response;
}

