/* ftpq: ftpq.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

int main (int argc, char **argv) {

	enum {HOST, PORT, LOGIN, PASSWD, ACCT, JOBID, DSTDIR, DSTFILE, MAIL, CONFIG, WAIT_SW, DONE} state = WAIT_SW;
	char cwd[1024];
	int dstfile = 0;
	int dstdir = 0;
	int filectr = 0;
	int arg_ctr = 0;

	/* Set defaults */
	gv.verbose = QUIET;
	gv.port = 21;
	gv.user = getpwuid (getuid ());
	gv.host = NULL;
	gv.srcpath = NULL;
	gv.dstpath = NULL;
	gv.login = NULL;
	gv.passwd = NULL;
	gv.rate = 0;
	gv.mail = 0;
	gv.mode = 0;	
	gv.order = ORDER_RND;
	gv.fileidx = 0;
	gv.acct = NULL;
	gv.transfer = 0L;
	gv.elapsed = 0L;

	/* No arguments - print usage and exit */
	if (argc == 1) {
		usage ();
		exit(OK);
	}

	/* Determine operating mode for this session */
	if (strcmp (argv[1], "put") == 0)
		gv.mode = MODE_PUT;
	else if (strcmp (argv[1], "add") == 0) {
		printf ("ftpq add is now obsolete. Please see man ftpq.\n\n");
		usage ();
		exit (AB_ARGERR);
	}
	else if (strcmp (argv[1], "del") == 0)
		gv.mode = MODE_DEL;
	else if (strcmp (argv[1], "get") == 0)
		gv.mode = MODE_GET;
	else if (strcmp (argv[1], "rmdir") == 0)
		gv.mode = MODE_RMDIR;
	else if (strcmp (argv[1], "mkdir") == 0)
		gv.mode = MODE_MKDIR;
	else if (strcmp (argv[1], "remove") == 0)
		gv.mode = MODE_REMOVE;
	else if (strcmp (argv[1], "list") == 0)
		gv.mode = MODE_LIST;
	else if (strcmp (argv[1], "run") == 0)
		gv.mode = MODE_RUN;
	else {
		usage ();
		exit (AB_ARGERR);
	}

	/* Process remaining arguments for add mode */
	if (gv.mode & (MODE_PUT|MODE_GET|MODE_MKDIR|MODE_RMDIR|MODE_DEL)) {
		arg_ctr = 2;
		while (arg_ctr < argc) {
			/* Identification switch */
			if (strcmp (argv[arg_ctr], "-u") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = LOGIN;
			}
			/* Config switch */
			else if (strcmp (argv[arg_ctr], "-c") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = CONFIG;
			}
			/* Password switch */
			else if (strcmp (argv[arg_ctr], "-p") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = PASSWD;
			}
			/* Account switch */
			else if (strcmp (argv[arg_ctr], "-a") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = ACCT;
			}
			/* Destination path switch */
			else if (strcmp (argv[arg_ctr], "-d") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = DSTDIR;
			}
			/* Destination file switch */
			else if (strcmp (argv[arg_ctr], "-f") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = DSTFILE;
			}
			/* Mail notification switch */
			else if (strcmp (argv[arg_ctr], "-m") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = MAIL;
			}
			/* Remote host switch */
			else if (strcmp (argv[arg_ctr], "-h") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = HOST;
			}
			/* Remote port switch */
			else if (strcmp(argv[arg_ctr], "-P") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = PORT;
			}
			else if (state != DONE) {
				switch (state) {
				/* Either local file or error */
				  case WAIT_SW:
					if (gv.host != NULL && gv.login != NULL) {
						gv.fileidx = arg_ctr;
						state = DONE;
					}
					else {
						printf ("Host or login not provided.\n");
						usage ();
						exit (AB_ARGERR);
					}
					break;
				  case HOST:
					if (gv.host != NULL)
						free (gv.host);
					gv.host = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv.host, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case ACCT:
					if (gv.acct != NULL)
						free (gv.acct);
					gv.acct = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv.acct, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case DSTFILE:
					dstfile = arg_ctr;
					state = WAIT_SW;
					break;
				  case DSTDIR:
					dstdir = arg_ctr;
					state = WAIT_SW;
					break;
				  case MAIL:
					switch (argv[arg_ctr][0]) {
					  case 'n':
						gv.mail = 0;
						break;
					  case 's':
						gv.mail = MAIL_SUCCESS;
						break;
					  case 'f':
						gv.mail = MAIL_FAILURE;
						break;
					  case 'a':
						gv.mail = MAIL_SUCCESS | MAIL_FAILURE;
						break;
					  default:
						printf ("ftpq: -m n|s|f|a \n");
						usage ();
						exit (AB_CASEERR);
					}
					state = WAIT_SW;
					break;
				  case LOGIN:
					if (gv.login != NULL)
						free (gv.login);
					gv.login = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv.login, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case PASSWD:
					if (gv.passwd != NULL)
						free (gv.passwd);
					gv.passwd = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv.passwd, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case CONFIG:
					if (read_config (argv, arg_ctr) != OK)
						exit (AB_ARGERR);
					state = WAIT_SW;
					break;
				  case PORT:
					gv.port = atoi (argv[arg_ctr]);
					if (gv.port < 1) {
						usage ();
						exit (AB_ARGERR);
					}
					state = WAIT_SW;
					break;
				/* Remaining args are multiple files */
				  case DONE:
					break;
				  default:
					printf ("Argh - it's gone horribly wrong!\n");
					exit (AB_ARGERR);
					break;
				}
			}
			arg_ctr++;
		}

		/* Args processed: must have host, login, file */
		if ((gv.host == NULL || gv.login == NULL || gv.fileidx == 0) || state != DONE) {
			printf ("Missing mandatory parameter:\n");
			printf ("host: %s / login: %s\n", gv.host, gv.login);
			usage ();
			exit (AB_ARGERR);
		}
		gv.filectr = argc - gv.fileidx;

		/* If multiple files and dstfile given, abort. */
		if (dstfile != 0 && gv.filectr > 1) {
			printf ("Can't specify -f with multiple files!\n");
			usage ();
			exit (AB_ARGERR);
		}

		/* If dstfile given in DEL, MKDIR or RMDIR mode, abort. */
		if (dstfile != 0 && (gv.mode & (MODE_DEL|MODE_RMDIR|MODE_MKDIR))) {
			printf ("Can't specify -f with ftpq del|mkdir|rmdir!\n");
			usage ();
			exit (AB_ARGERR);
		}

		/* If dstdir given in MKDIR or RMDIR mode, abort. */
		if (dstdir != 0 && (gv.mode & (MODE_RMDIR|MODE_MKDIR))) {
			printf ("Can't specify -d with ftpq mkdir|rmdir!\n");
			usage ();
			exit (AB_ARGERR);
		}

		/* Let's go! */
		getcwd (cwd, 1023);
		while (filectr < gv.filectr) {
			/* First, tidy up srcpath */
			if (gv.mode & (MODE_PUT|MODE_GET)) {
				if (argv[filectr + gv.fileidx][0] != '/' && gv.mode & MODE_PUT) {
					gv.srcpath = memalloc (strlen (cwd) + strlen (argv[filectr + gv.fileidx]) + 2);
					sprintf (gv.srcpath, "%s/%s", cwd, argv[filectr + gv.fileidx]);
				}
				else {
					gv.srcpath = memalloc (strlen (argv[filectr + gv.fileidx]) + 2);
					sprintf (gv.srcpath, "%s", argv[filectr + gv.fileidx]);
				}
			}

			/* Now to build up dstpath... */
			/* First, strip leading / from -f if -d given */
			if (dstdir != 0 && dstfile != 0) {  
				while (argv[dstfile][0] == '/' && strlen (argv[dstfile]) > 1)
					argv[dstfile] = &argv[dstfile][1];
			}			

			/* Now, strip trailing / from -f and -d */
			if (dstdir != 0) { 
				while (argv[dstdir][strlen (argv[dstdir]) - 1] == '/' && strlen (argv[dstdir]) > 1)
					argv[dstdir][strlen (argv[dstdir]) - 1] = '\0';
			} 
			if (dstfile != 0) { 
				while (argv[dstfile][strlen (argv[dstfile]) - 1] == '/' && strlen (argv[dstfile]) > 1)
					argv[dstfile][strlen (argv[dstfile]) - 1] = '\0';
			} 
			if (dstdir != 0) {
				if (dstfile != 0) {
					gv.dstpath = memalloc (strlen (argv[dstdir]) + strlen (argv[dstfile]) + 2); 
					sprintf (gv.dstpath, "%s/%s", argv[dstdir], argv[dstfile]);
				}
				else if (gv.mode & MODE_DEL) {
					gv.dstpath = memalloc (strlen (argv[dstdir]) + strlen (argv[filectr + gv.fileidx]) + 2); 
					sprintf (gv.dstpath, "%s/%s", argv[dstdir], argv[filectr + gv.fileidx]);
				}
				else { 
					gv.dstpath = memalloc (strlen (argv[dstdir]) + strlen (extract_file (argv[filectr + gv.fileidx])) + 2); 
					sprintf (gv.dstpath, "%s/%s", argv[dstdir], extract_file (argv[filectr + gv.fileidx]));
				}
			}
			else {
				if (dstfile != 0) { 
					gv.dstpath = memalloc (strlen (argv[dstfile]) + 1); 
					sprintf (gv.dstpath, "%s", argv[dstfile]);
				}
				else {
					if (gv.mode & (MODE_PUT|MODE_GET)) {
						gv.dstpath = memalloc (strlen (extract_file (argv[filectr + gv.fileidx])) + 1); 
						sprintf (gv.dstpath, "%s", extract_file (argv[filectr + gv.fileidx]));
					}
					else {
						gv.dstpath = memalloc (strlen (argv[filectr + gv.fileidx]) + 1); 
						sprintf (gv.dstpath, "%s", argv[filectr + gv.fileidx]);
					}
				}
			}

			/* Now go and do it... */
			add_to_queue (argv);
			
			free (gv.dstpath);
			free (gv.srcpath);
			filectr++;
		}
	}

	else if (gv.mode & MODE_LIST)
		 parse_svrs (QLIST); 

	else if (gv.mode & MODE_RUN) {
		arg_ctr = 2;
		while (arg_ctr < argc) {
			if (strcmp (argv[arg_ctr], "-v") == 0)
				gv.verbose = VERBOSE;
			else if (strcmp (argv[arg_ctr], "-o") == 0)
				gv.order = ORDER_CHRONO;
			else if (strcmp (argv[arg_ctr], "-r") == 0) {
				gv.rate = atoi (argv[++arg_ctr]);
				if (gv.rate < 1000) {
					printf ("ftpq: silly rate.\n");
					usage ();
					exit (AB_ARGERR);
				}
#ifndef HAVE_SLEEP
#ifndef HAVE_USLEEP
				printf ("ftpq: can't use rate limiting - no sleep function on this system!\n");
				usage ();
				exit (AB_ARGERR);
#endif /* don't HAVE_USLEEP */
#endif /* don't HAVE_SLEEP  */
			}	
			else {
				usage ();
				exit (AB_ARGERR);
			}
			arg_ctr++;
		}
		parse_svrs (QRUN); 
	}

	/* else if (gv.mode == MODE_REMOVE)
		delete_from_queue (argc, argv); */

	return OK;
}

