/* ftpq: ftpq.h

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* includes */
#include "../config.h"
#ifdef HAVE_CTYPE_H
	#include <ctype.h>
#endif
#ifdef HAVE_DIRENT_H
	#include <dirent.h>
#endif
#ifdef HAVE_ERRNO_H
	#include <errno.h>
#endif
#ifdef HAVE_NETDB_H
	#include <netdb.h>
#endif
#ifdef HAVE_NETINET_IN_H
	#include <netinet/in.h>
#endif
#ifdef HAVE_PWD_H
	#include <pwd.h>
#endif
#ifdef HAVE_STDIO_H
	#include <stdio.h>
#endif
#ifdef HAVE_STDLIB_H
	#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
	#include <string.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
	#include <sys/socket.h>
#endif
#ifdef HAVE_SYS_STAT_H
	#include <sys/stat.h>
#endif
#ifdef HAVE_SYS_TIME_H
	#include <sys/time.h>
#endif
#ifdef HAVE_SYS_TYPES_H
	#include <sys/types.h>
#endif
#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif

/* useful defined constants */
#define SPOOLDIR    "/tmp/ftpq/"   /* must end in /. Check Makefile, too. */
#define HOSTRESP    2048           /* ftp server response buffer, bytes */
#define BUFSIZE     2048           /* general purpose global buffer */
#define SMTP_PORT     25

/* internal defined constants */
#define VERSION     1.11

#define MAIL_SUCCESS   0x01
#define MAIL_FAILURE   0x02

#define MODE_PUT       0x0001
#define MODE_GET       0x0002
#define MODE_DEL       0x0004
#define MODE_RMDIR     0x0008
#define MODE_MKDIR     0x0010 
#define MODE_LIST      0x0020
#define MODE_RUN       0x0040
#define MODE_REMOVE    0x0080

#define FO_READ        0x01
#define FO_WRITE       0x02
#define FO_NEED        0x04
#define FO_NONEED      0x08 

#define ORDER_RND      1
#define ORDER_CHRONO   2

#define OK             0
#define AB_NOFILE      1
#define AB_NOPERM      2
#define AB_FILEERR     3
#define AB_MALLOC      4
#define AB_CASEERR     5
#define AB_NOSPOOL     6
#define AB_ARGERR      7
#define NOCOPY       201
#define COPY         202
#define VERBOSE      203
#define QUIET        204
#define NOLFILE      301
#define POS_PRELIM   401
#define POS_COMPLETE 402
#define POS_INTER    403
#define NEG_TRANS    404
#define NEG_PERM     405
#define RESP_ERROR   406
#define NOLOGIN      501
#define NOCWDROOT    502
#define NOCWD        503
#define NOPASV       504
#define NOTYPEI      505
#define NOMODES      506
#define NOSTOR       507
#define NOSERVER     508
#define NORETR       509
#define NODELE       510
#define NOFTPQRC     601
#define NOSECTION    602
#define MAILERR      701
#define RESP         801
#define NORESP       802
#define SEC_FAIL     901
#define SEC_PASS     902
#define QRUN        1001
#define QLIST       1002

/* Data structures and type definitions */

struct global {
	char *host;
	char *login;
	char *passwd;
	char *acct;
	char *dstpath;
	char *srcpath;
	int fileidx;
	int filectr;
	int mode;
	int order;
	int mail;
	int port;
	int verbose;
	int rate;
	long elapsed;
	long transfer;
	struct passwd *user;
	struct sockaddr_in local_ip;
};

/* function declarations */

#ifdef __STRICT_ANSI__
  /* These functions need declaring when gcc -ansi is used. */
  void usleep                 (unsigned long);
  int gethostname             (char *, size_t);
#endif /* __STRICT_ANSI__ */

void usage                    (void);
FILE *fileopen                (char *, int);
void *memalloc                (size_t);
int read_config               (char **, int);
int fileputc                  (int, FILE *);
int get_response              (int);
int data_connect              (int, int, int, int, int, int);
int send_cmd                  (char *, int, int);
int category                  (int);
int get_socket_out            (char *, int);
int get_socket_in             (int);
long next_jobid               (void);
int parse_svrs                (int);
int parse_users               (char *, int, int);
int parse_jobs                (char *, int, int, int);
int qprocess                  (char *, int, int);	
int usr_login                 (char *, char *, int); 
int svr_login                 (char *, int);
int svr_logout                (int);
int ftp_user                  (char *, int);
int ftp_pass                  (char *, int);
int ftp_acct                  (char *, int);
int ftp_cwd                   (char *, int);
int ftp_cdup                  (int);
int ftp_rein                  (int);
int ftp_quit                  (int);
int ftp_port                  (int, int, int, int, int, int, int);
int ftp_pasv                  (int);
int ftp_modes                 (int);
int ftp_typei                 (int);
int ftp_struf                 (int); 
int ftp_retr                  (char *, int);
int ftp_stor                  (char *, int);
int ftp_dele                  (char *, int);
int ftp_mkd                   (char *, int);
int ftp_rmd                   (char *, int);
int ftp_allo                  (long, int);
char *extract_file            (char *);
int path_depth                (char *, int);
int add_to_queue              (char **);
int cnt_subdirs               (char *);
int send_data                 (int);
int get_data                  (int);
int do_get                    (int);
int do_del                    (int);
int do_put                    (int);
int do_rmdir                  (int);
int do_mkdir                  (int);
char *my_inet_ntoa            (struct in_addr);     


/* global variables */
struct global gv;
char hostresp[HOSTRESP];
char buffer[BUFSIZE];
