/* ftpq: parse.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

int parse_svrs (int mode) {

	DIR *sdir = NULL;
	struct dirent *svr;
	struct stat fdets;
	int port, sockfd;
	char *svrname = NULL;
	char *svrdir = NULL;

	sdir = opendir (SPOOLDIR);

	if (sdir == NULL) {
		printf ("ftpq: couldn't access spool directory %s!\n", SPOOLDIR);
		exit (AB_NOSPOOL);
	}
	
	while ((svr = readdir (sdir)) != NULL) {
		if (strcmp (svr->d_name, ".") != 0 && strcmp (svr->d_name, "..") != 0) {
			svrdir = memalloc (strlen (SPOOLDIR) + strlen (svr->d_name) + 2);
			sprintf (svrdir, "%s%s", SPOOLDIR, svr->d_name);

			stat (svrdir, &fdets);
			if (S_ISDIR (fdets.st_mode)) {
				svrname = (char *) strtok (svr->d_name, "+");
				port = atoi (strtok (NULL, "+"));
				if (mode == QRUN) {
					sockfd = svr_login (svrname, port);
					if (sockfd != 0) {
						parse_users (svrdir, sockfd, mode);
						svr_logout (sockfd);
						rmdir (svrdir);
					}
					close (sockfd);
				}
				else if (mode == QLIST) {
					parse_users (svrdir, 0, mode);	
				}
			}	
			free (svrdir);
		}
	}

	return OK;
}

int parse_users (char *sdir, int sockfd, int mode) {

	DIR *udir = NULL;
	struct dirent *usr;
	struct stat fdets;
	struct stat ddets;
	char *jobdir = NULL;
	char *usrdir = NULL; 
	char *usrdet = NULL;
	int finished = 0;

	udir = opendir (sdir);

	while ((usr = readdir (udir)) != NULL) {
		if (strcmp (usr->d_name, ".") != 0 && strcmp (usr->d_name, "..") != 0) {
			usrdir = memalloc (strlen (sdir) + strlen (usr->d_name) + 2);
			sprintf (usrdir, "%s/%s", sdir, usr->d_name);
			stat (usrdir, &fdets);
			if (S_ISDIR (fdets.st_mode)) {

				if (mode == QRUN) { 
					usr_login (usrdir, usr->d_name, sockfd);
					finished = 0;
				}

				jobdir = memalloc (strlen (usrdir) + 7);

				sprintf (jobdir, "%s/home", usrdir);
				if (stat (jobdir, &ddets) != -1) {
					parse_jobs (jobdir, sockfd, mode, 0);
					if (mode == QRUN) 
						finished += rmdir (jobdir);
				}

				sprintf (jobdir, "%s/root", usrdir);
				if (stat (jobdir, &ddets) != -1) {
					ftp_cwd ("/", sockfd);
					parse_jobs (jobdir, sockfd, mode, 0);
					if (mode == QRUN)
						finished += rmdir (jobdir);
				}
				free (jobdir);
				if (mode == QRUN && finished == 0) {
					usrdet = memalloc (strlen (usrdir) + 6);
					sprintf (usrdet, "%s/user", usrdir);
					unlink (usrdet);
					free (usrdet);
					rmdir (usrdir);
				} 
			}
			free (usrdir);
		}
	}
	return OK;
}

	

/* parse_jobs() reads all of the jobs in a directory. If a subdirectory
   is found, it recurses into it. */

int parse_jobs (char *dname, int sockfd, int mode, int reclvl) {

	DIR *qdir = NULL;
	struct dirent *cfile;
	struct stat fdets;
	char *jname = NULL;
	int response;

	qdir = opendir (dname); 

	while ((cfile = readdir (qdir)) != NULL) {
		if (strcmp (cfile->d_name, ".") != 0 && strcmp (cfile->d_name, "..") != 0) {
			jname = memalloc (strlen (dname) + strlen (cfile->d_name) + 2);
			sprintf (jname, "%s/%s", dname, cfile->d_name);
			stat (jname, &fdets);		
			if (S_ISDIR (fdets.st_mode)) {
				if (mode == QRUN) {
					response = ftp_cwd (cfile->d_name, sockfd);
					if (category (response) == NEG_PERM) {
						response = ftp_mkd (cfile->d_name, sockfd);
						if (category (response) == POS_COMPLETE) {
							response = ftp_cwd (cfile->d_name, sockfd);
							if (category (response) != POS_COMPLETE)
								return NOCWD;
						}
						else
							return NOCWD;
					}
				}
				parse_jobs (jname, sockfd, mode, reclvl + 1);
				if (mode == QRUN) {
					response = ftp_cdup (sockfd);
					rmdir (dname);
				}
			}
			else
				qprocess (jname, sockfd, mode);
		}
	}
	return OK;
}

int qprocess (char *qfile, int sockfd, int mode) {

	FILE *jptr;
	char line [1024];
	char buf [1024]; 
	long jobid = 0; 
	char *locpath = NULL;
	char *rempath = NULL;

	
	jptr = fileopen (qfile, FO_READ | FO_NONEED);

	if (jptr == NULL)
		return OK;

	free (gv.srcpath);
	free (gv.dstpath);

	gv.mode = 0;

	while (fgets (line, 1023, jptr) != NULL) {	
		if (sscanf (line, "jo: %ld", &jobid));
		else if (sscanf (line, "lp: %s", buf)) {
			locpath = memalloc (strlen (buf) + 1);
			strcpy (locpath, buf);
		}
		else if (sscanf (line, "rp: %s", buf)) {
			rempath = memalloc (strlen (buf) + 1);
			strcpy (rempath, buf);
		}
		else if (sscanf (line, "mo: %d", &gv.mail)); 
		else if (sscanf (line, "md: %d", &gv.mode)); 
	}

	fclose (jptr);

	if (mode == QRUN) {
		switch (gv.mode) {
		  case MODE_PUT:
			gv.srcpath = locpath;
			gv.dstpath = rempath;
			if (do_put (sockfd) == OK) {
				unlink (qfile);
			}	
			break;
		  case MODE_GET:
			gv.srcpath = rempath;
			gv.dstpath = locpath;
			if (do_get (sockfd) == OK) {
				unlink (qfile);
			}	
			break;
		  case MODE_DEL:
			gv.dstpath = rempath;
			if (do_del (sockfd) == OK) {
				unlink (qfile);
			}	
			break;
		  case MODE_RMDIR:
			gv.dstpath = rempath;
			if (do_rmdir (sockfd) == OK) {
				unlink (qfile);
			}	
			break;
		  case MODE_MKDIR:
			gv.dstpath = rempath;
			if (do_mkdir (sockfd) == OK) {
				unlink (qfile);
			}	
			break;
		  default:
			printf ("Dodgy job file! Aborting!\n");
			exit (AB_CASEERR);
		} 
	}

	return OK;
}
		 
