/* ftpq: socket.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

int get_socket_out (char *host, int port) {

	int sockfd, length;
	struct hostent *he;
	struct sockaddr_in their_addr; /* connector's address information */

	if ((he = gethostbyname (host)) == NULL) {  /* get the host info */
		printf ("\nftpq: could not get host information! (off-line?)\n");
		return 0;
	}

	if ((sockfd = socket (AF_INET, SOCK_STREAM, 0)) == -1) {
		printf ("\nftpq: could not get socket!\n");
		return 0;
	}

	their_addr.sin_family = AF_INET;
	their_addr.sin_port = htons(port);
	their_addr.sin_addr = *((struct in_addr *)he->h_addr);
	memset (&(their_addr.sin_zero), '\0', 8);

	if (connect(sockfd, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) == -1) {
		printf ("\nftpq: could not connect to socket!\n");
		return 0;
	}

	length = sizeof (gv.local_ip);
	getsockname (sockfd, (struct sockaddr *) &gv.local_ip, (socklen_t *) &length);

	return sockfd;

}


int get_socket_in (int sfd) {

	int sockfd, infd, sin_size, length, response;
	int p[6];
	char hostname[256];
	char *ipaddr;
	struct hostent *host;
	struct sockaddr_in my_addr;
	struct sockaddr_in their_addr;

	if ((sockfd = socket (AF_INET, SOCK_STREAM, 0)) == -1) {
		printf ("ftpq: couldn't get socket!\n");
		return 0;
	}

	if (gethostname (hostname, 255) == -1) {
		printf ("ftpq: couldn't get local host name!\n");
		return 0;
	}

	host = gethostbyname (hostname);

	my_addr.sin_family = AF_INET;
	my_addr.sin_port = 0;
	my_addr.sin_addr = gv.local_ip.sin_addr;
	memset (&(my_addr.sin_zero), '\0', 8);

	if (bind (sockfd, (struct sockaddr *) &my_addr, sizeof (struct sockaddr)) == -1) {
		printf ("ftpq: couldn't bind to socket!\n");
		return 0;
	}
	
	length = sizeof (my_addr);
	getsockname (sockfd, (struct sockaddr *) &my_addr, (socklen_t *) &length);
	listen (sockfd, 10);

	ipaddr = my_inet_ntoa (my_addr.sin_addr);

	p[0] = atoi (strtok (ipaddr, "."));
	p[1] = atoi (strtok (NULL, "."));
	p[2] = atoi (strtok (NULL, "."));
	p[3] = atoi (strtok (NULL, "."));
	p[4] = (htons (my_addr.sin_port) >> 8) & 0x00ff;
	p[5] = htons (my_addr.sin_port) & 0x00ff;

	free (ipaddr);

	response = ftp_port (p[0], p[1], p[2], p[3], p[4], p[5], sfd); 
	if (response != 200)
		return 0;

	ftp_retr (extract_file (gv.srcpath), sfd);	
	sin_size = sizeof (struct sockaddr_in);	
	infd = accept (sockfd, (struct sockaddr *) &their_addr, (socklen_t *) &sin_size);
	close (sockfd);
	
	return infd;
}


/* my_inet_ntoa is my implementation of inet_ntoa, to get round some header */
/* problems with Solaris. Remember to free the char * you allocate! */

char *my_inet_ntoa (struct in_addr in) {

	char *outbuf = NULL;
	long ip;
	int d[4];

	outbuf = memalloc (16);
	ip = htonl (in.s_addr);
	
	d[0] = (int) ((ip >> 24) & 0x00ff); 
	d[1] = (int) ((ip >> 16) & 0x00ff); 
	d[2] = (int) ((ip >> 8) & 0x00ff); 
	d[3] = (int) (ip & 0x00ff); 
	
	sprintf (outbuf, "%d.%d.%d.%d", d[0], d[1], d[2], d[3]);

	return outbuf;
}
